@@ -1,5 +1,6 @@
 #include <Rcpp.h>
 #include "shared.h"
+// [[Rcpp::plugins(cpp11)]]
 
 using std::pow;
 using std::sqrt;
@@ -8,13 +9,7 @@ using std::exp;
 using std::log;
 using std::floor;
 using std::ceil;
-using std::sin;
-using std::cos;
-using std::tan;
-using std::atan;
-using Rcpp::IntegerVector;
 using Rcpp::NumericVector;
-using Rcpp::NumericMatrix;
 
 
 /*
@@ -30,41 +25,41 @@ using Rcpp::NumericMatrix;
  * 
  */
 
-double pdf_wald(double x, double mu, double lambda) {
+inline double pdf_wald(double x, double mu, double lambda,
+                       bool& throw_warning) {
   if (ISNAN(x) || ISNAN(mu) || ISNAN(lambda))
-    return NA_REAL;
+    return x+mu+lambda;
   if (mu <= 0.0 || lambda <= 0.0) {
-    Rcpp::warning("NaNs produced");
+    throw_warning = true;
     return NAN;
   }
-  if (x <= 0.0 || std::isinf(x))
+  if (x <= 0.0 || !R_FINITE(x))
     return 0.0;
   return sqrt(lambda/(2.0*PI*pow(x, 3.0))) *
          exp((-lambda*pow(x-mu, 2.0))/(2.0*pow(mu, 2.0)*x));
 }
 
-double cdf_wald(double x, double mu, double lambda) {
+inline double cdf_wald(double x, double mu, double lambda,
+                       bool& throw_warning) {
   if (ISNAN(x) || ISNAN(mu) || ISNAN(lambda))
-    return NA_REAL;
+    return x+mu+lambda;
   if (mu <= 0.0 || lambda <= 0.0) {
-    Rcpp::warning("NaNs produced");
+    throw_warning = true;
     return NAN;
   }
   if (x <= 0.0)
     return 0.0;
-  if (x == INFINITY)
+  if (x == R_PosInf)
     return 1.0;
   return Phi(sqrt(lambda/x)*(x/mu-1.0)) +
          exp((2.0*lambda)/mu) *
          Phi(-sqrt(lambda/x)*(x/mu+1.0));
 }
 
-double rng_wald(double mu, double lambda) {
-  if (ISNAN(mu) || ISNAN(lambda))
+inline double rng_wald(double mu, double lambda, bool& throw_warning) {
+  if (ISNAN(mu) || ISNAN(lambda) || mu <= 0.0 || lambda <= 0.0) {
+    throw_warning = true;
     return NA_REAL;
-  if (mu <= 0.0 || lambda <= 0.0) {
-    Rcpp::warning("NaNs produced");
-    return NAN;
   }
   double u, x, y, z;
   u = rng_unif();
@@ -84,21 +79,27 @@ NumericVector cpp_dwald(
     const NumericVector& x,
     const NumericVector& mu,
     const NumericVector& lambda,
-    bool log_prob = false
+    const bool& log_prob = false
   ) {
   
-  int n  = x.length();
-  int nm = mu.length();
-  int nl = lambda.length();
-  int Nmax = Rcpp::max(IntegerVector::create(n, nm, nl));
+  int Nmax = std::max({
+    x.length(),
+    mu.length(),
+    lambda.length()
+  });
   NumericVector p(Nmax);
   
+  bool throw_warning = false;
+  
   for (int i = 0; i < Nmax; i++)
-    p[i] = pdf_wald(x[i % n], mu[i % nm], lambda[i % nl]);
+    p[i] = pdf_wald(GETV(x, i), GETV(mu, i),
+                    GETV(lambda, i), throw_warning);
   
   if (log_prob)
-    for (int i = 0; i < Nmax; i++)
-      p[i] = log(p[i]);
+    p = Rcpp::log(p);
+  
+  if (throw_warning)
+    Rcpp::warning("NaNs produced");
   
   return p;
 }
@@ -109,25 +110,31 @@ NumericVector cpp_pwald(
     const NumericVector& x,
     const NumericVector& mu,
     const NumericVector& lambda,
-    bool lower_tail = true, bool log_prob = false
+    const bool& lower_tail = true,
+    const bool& log_prob = false
   ) {
   
-  int n  = x.length();
-  int nm = mu.length();
-  int nl = lambda.length();
-  int Nmax = Rcpp::max(IntegerVector::create(n, nm, nl));
+  int Nmax = std::max({
+    x.length(),
+    mu.length(),
+    lambda.length()
+  });
   NumericVector p(Nmax);
   
+  bool throw_warning = false;
+  
   for (int i = 0; i < Nmax; i++)
-    p[i] = cdf_wald(x[i % n], mu[i % nm], lambda[i % nl]);
+    p[i] = cdf_wald(GETV(x, i), GETV(mu, i),
+                    GETV(lambda, i), throw_warning);
   
   if (!lower_tail)
-    for (int i = 0; i < Nmax; i++)
-      p[i] = 1.0 - p[i];
+    p = 1.0 - p;
   
   if (log_prob)
-    for (int i = 0; i < Nmax; i++)
-      p[i] = log(p[i]);
+    p = Rcpp::log(p);
+  
+  if (throw_warning)
+    Rcpp::warning("NaNs produced");
   
   return p;
 }
@@ -135,17 +142,21 @@ NumericVector cpp_pwald(
 
 // [[Rcpp::export]]
 NumericVector cpp_rwald(
-    const int n,
+    const int& n,
     const NumericVector& mu,
     const NumericVector& lambda
   ) {
   
-  int nm = mu.length();
-  int nl = lambda.length();
   NumericVector x(n);
   
+  bool throw_warning = false;
+  
   for (int i = 0; i < n; i++)
-    x[i] = rng_wald(mu[i % nm], lambda[i % nl]);
+    x[i] = rng_wald(GETV(mu, i), GETV(lambda, i),
+                    throw_warning);
+  
+  if (throw_warning)
+    Rcpp::warning("NAs produced");
   
   return x;
 }
